
import designs
import factors
import models

from ui import anova1,anova1rm
from ui import anova2,anova2nested,anova2rm,anova2onerm
from ui import anova3,anova3nested,anova3rm,anova3tworm,anova3onerm
