

'''
spm1D: A Python package for 1D Statistical Parametric Mapping

Useful references include:

[1] Statistical Parametric Mapping: The Analysis of Functional Brain Images
Karl Friston, John Ashburner, Stefan Kiebel, Thomas Nichols (Editors)
Academic Press, 2006

[2] Wellcome Trust Centre for Neuroimaging SPM homepage:
www.fil.ion.ucl.ac.uk/spm/

[3] Pataky TC (2010). Generalized n-dimensional field analysis using
statistical parametric mapping. Journal of Biomechanics.


Copyright (C) 2015  Todd Pataky
Version: 0.3.1.5 (2015/12/09)
'''


__version__ = '0.3.1.5 (2015/12/09)'


__all__ = ['io', 'plot', 'stats', 'util']


import data, io, plot, stats, util



